"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Application = void 0;
const class_transformer_1 = require("class-transformer");
const rules_1 = require("./rules");
const version_1 = require("./version");
var SaveBy;
(function (SaveBy) {
    SaveBy[SaveBy["AppName"] = 0] = "AppName";
    SaveBy[SaveBy["Applications"] = 1] = "Applications";
})(SaveBy || (SaveBy = {}));
class Application {
    constructor(init) {
        Object.assign(this, init);
        this._keyBy = SaveBy.AppName;
    }
    static async UpdateDefaultRule(opts) {
        const { dbManager, key } = opts;
        const rules = await rules_1.Rules.Load({ dbManager, key });
        const defaultRule = rules.RuleSet.default;
        defaultRule.SemVer = key.SemVer;
        await rules.Save(dbManager);
    }
    static async GetVersionsAndRules(opts) {
        const { key, dbManager } = opts;
        // Get all versions and rules for an app
        // Note: versions are moved out of this key as they become inactive
        // There should be less than, say, 100 versions per app
        const versionTask = version_1.Version.LoadVersions({ dbManager, key });
        const rulesTask = rules_1.Rules.Load({ dbManager, key });
        await Promise.all([versionTask, rulesTask]);
        return {
            Versions: await versionTask,
            Rules: await rulesTask,
        };
    }
    static async Load(opts) {
        const { key, dbManager } = opts;
        const { Item } = await dbManager.ddbDocClient.get({
            TableName: dbManager.tableName,
            Key: { PK: `appName#${key.AppName}`.toLowerCase(), SK: 'application' },
        });
        const record = (0, class_transformer_1.plainToInstance)(Application, Item);
        return record;
    }
    static async LoadAllApps(dbManager) {
        const { Items } = await dbManager.ddbDocClient.query({
            TableName: dbManager.tableName,
            KeyConditionExpression: 'PK = :pkval',
            ExpressionAttributeValues: {
                ':pkval': 'applications',
            },
        });
        const records = [];
        if (Items !== undefined) {
            for (const item of Items) {
                const record = (0, class_transformer_1.plainToInstance)(Application, item);
                records.push(record);
            }
        }
        return records;
    }
    get DbStruct() {
        return {
            PK: this.PK,
            SK: this.SK,
            AppName: this.AppName,
            DisplayName: this.DisplayName,
        };
    }
    async Save(dbManager) {
        // TODO: Validate that all the fields needed are present
        // Save under specific AppName key
        this._keyBy = SaveBy.AppName;
        const taskByName = dbManager.ddbDocClient.put({
            TableName: dbManager.tableName,
            Item: this.DbStruct,
        });
        // Save under all Applications key
        this._keyBy = SaveBy.Applications;
        const taskByApplications = dbManager.ddbDocClient.put({
            TableName: dbManager.tableName,
            Item: this.DbStruct,
        });
        await Promise.all([taskByName, taskByApplications]);
        // Await the tasks so they can throw / complete
        await taskByName;
        await taskByApplications;
    }
    get PK() {
        switch (this._keyBy) {
            case SaveBy.Applications:
                return 'applications';
            case SaveBy.AppName:
                return `appName#${this.AppName}`.toLowerCase();
            default:
                throw new Error('Missing SaveBy handler');
        }
    }
    get SK() {
        switch (this._keyBy) {
            case SaveBy.Applications:
                return `appName#${this.AppName}`.toLowerCase();
            case SaveBy.AppName:
                return 'application';
            default:
                throw new Error('Missing SaveBy handler');
        }
    }
    get AppName() {
        return this._appName;
    }
    set AppName(value) {
        this._appName = value.toLowerCase();
    }
    get DisplayName() {
        return this._displayName;
    }
    set DisplayName(value) {
        this._displayName = value;
    }
}
exports.Application = Application;
//# sourceMappingURL=application.js.map